#include <allegro5/allegro_primitives.h>
#include "globals.hpp"
#include "game.hpp"
#include "testgame.hpp"
#include "state.hpp"
#include "conditioncases.hpp"
#include "stategrid.hpp"

void game_mainline(GameVars& game, GlobalVars& globals)
{
   bool go_on = true;
   double in;//Initial time: point of reference

   //!!For testing purposes only
   double gtu(0.0);//Graphics time used; time used by graphics
   double ltu(0.0);//Logic time used
   double gpl;//Graphics time per logic time
   TestGame tgame(globals.queue, gtu, ltu, gpl, globals.gdt, true);
   StateGrid g;
   State *upbutton_up, *upbutton_down;
   upbutton_up = g.create_state();
   upbutton_down = g.create_state();
   CertainButtonUp* upgoes_up = new CertainButtonUp(ALLEGRO_KEY_UP);
   CertainButtonDown* upgoes_down = new CertainButtonDown(ALLEGRO_KEY_UP);
   upbutton_down->add_condition(State::START_OTHER|State::LEAVE_SELF, upgoes_up, upbutton_up);
   upbutton_up->add_condition(State::START_OTHER|State::LEAVE_SELF, upgoes_down, upbutton_down);
   upbutton_up->enter();

   AnyTimerTicks* qc_response_time = new AnyTimerTicks(1000);
   AnyTimerTicks* effect_duration = new AnyTimerTicks(500);
   CertainButtonDown* downgoes_down = new CertainButtonDown(ALLEGRO_KEY_DOWN);
   CertainButtonDown* leftgoes_down = new CertainButtonDown(ALLEGRO_KEY_LEFT);
   CertainButtonDown* rightgoes_down = new CertainButtonDown(ALLEGRO_KEY_RIGHT);
   CertainButtonUp* downgoes_up = new CertainButtonUp(ALLEGRO_KEY_DOWN);
   //CertainButtonUp* leftgoes_up = new CertainButtonUp(ALLEGRO_KEY_LEFT);
   CertainButtonUp* rightgoes_up = new CertainButtonUp(ALLEGRO_KEY_RIGHT);
   State* sqc_0 = g.create_state();
   State* sdown = g.create_state();
   State* sdown_qc = g.create_state();
   State* sdownright = g.create_state();
   State* sdownright_qc = g.create_state();
   State* seffect = g.create_state();
   State* sright = g.create_state();
   sqc_0         -> add_condition(State::START_OTHER|State::LEAVE_SELF, downgoes_down, sdown_qc);
   sdown         -> add_condition(State::START_OTHER|State::LEAVE_SELF, downgoes_up, sqc_0);//x3
   sdown_qc      -> add_condition(State::START_OTHER|State::LEAVE_SELF, downgoes_up->alias(), sqc_0);
   sdown_qc      -> add_condition(State::START_OTHER|State::LEAVE_SELF, leftgoes_down, sdown);
   sdown_qc      -> add_condition(State::START_OTHER|State::LEAVE_SELF, qc_response_time, sdown);//x1
   sdown_qc      -> add_condition(State::START_OTHER|State::LEAVE_SELF, rightgoes_down, sdownright_qc);
   sdownright    -> add_condition(State::START_OTHER|State::LEAVE_SELF, downgoes_up->alias(), sright);
   sdownright    -> add_condition(State::START_OTHER|State::LEAVE_SELF, rightgoes_up, sdown);//x2
   sdownright_qc -> add_condition(State::START_OTHER|State::LEAVE_SELF, downgoes_up->alias(), seffect);
   sdownright_qc -> add_condition(State::START_OTHER|State::LEAVE_SELF, qc_response_time->alias(), sdownright_qc);
   sdownright_qc -> add_condition(State::START_OTHER|State::LEAVE_SELF, rightgoes_up->alias(), sdown);
   seffect       -> add_condition(State::START_OTHER|State::LEAVE_SELF, effect_duration, sdown);
   sright        -> add_condition(State::START_OTHER|State::LEAVE_SELF, rightgoes_up->alias(), sqc_0);
   sqc_0->enter();
   in = al_current_time();
   while (go_on)
   {
      al_wait_for_event(globals.queue, &globals.event);
      in = al_current_time();
      do {
         if(globals.event.type == ALLEGRO_EVENT_KEY_DOWN && globals.event.keyboard.keycode == ALLEGRO_KEY_ESCAPE){
            go_on = false;
            globals.gamestate = GlobalVars::Quit;
         }
         g.accept(&globals.event);

         tgame.logic(&globals.event);

      } while(al_get_next_event(globals.queue, &globals.event) && go_on && al_current_time() - in < globals.max_logic_time);
      ltu = al_current_time() - in;

      in = al_current_time();
      tgame.draw();
      if(upbutton_down->is_active())
         al_draw_filled_rectangle(300.0f, 100.0f, 330.0f, 164.0f, al_map_rgb_f(0.8f, 1, 0.8f));
      if(upbutton_up->is_active())
         al_draw_circle(315.0f, 132.0f, 24.0f, al_map_rgb_f(0.8f, 0.5f, 0.5f), 8.0f);
      if(seffect->is_active())
         al_draw_filled_circle(370.0f, 132.0f, 24.0f, al_map_rgb_f(0.8f, 0.5f, 0.5f));
      gtu = al_current_time() - in;
      al_flip_display();
      al_clear_to_color(al_map_rgb(0.0f, 0.0f, 0.0f));
      gpl = gtu/ltu;
      if( !finite(gpl) ){
         gpl = 0.0;
      }
   }
   //temp lol
   return;
}
