#include <allegro5/allegro5.h>
#include <map>
#include <stack>
#include <cmath>

class TestGame {
   private:
      double &gtu, &ltu, &gpl, &dt;
      double time, to_rest;
      bool log_not_lin;
      ALLEGRO_EVENT_SOURCE esrc;
      #define CUSTOM_EVENT_DRAWN     ALLEGRO_GET_EVENT_TYPE('t', 'c', 'e', 'd')
      ALLEGRO_EVENT drawn;
      class FunktionBar{
         //draws a bar from Rechts nach Links
         private:
            std::map<unsigned int, ALLEGRO_COLOR> overkill_map;
            std::stack<ALLEGRO_COLOR> bar_stuecken;
            ALLEGRO_COLOR other;
            const float x_pos, y_pos, line_size, size;
         public:
            void draw() {
               for (float start = x_pos; !bar_stuecken.empty(); bar_stuecken.pop()) {
                  al_draw_line(start, y_pos, start, y_pos+size, bar_stuecken.top(), line_size);
                  start -= line_size;
               }
            }

            void set(ALLEGRO_EVENT* e){
               std::map<unsigned int, ALLEGRO_COLOR>::iterator i = overkill_map.find(e->type);
               if( i == overkill_map.end()){
                  bar_stuecken.push(other);
               }
               else {
                  bar_stuecken.push(i->second);
               }
            }

            FunktionBar(float xpos, float ypos, float lsize)
               : other(al_map_rgb_f(1.0f, 0.0f, 0.0f)), x_pos(xpos), y_pos(ypos), line_size(lsize), size(40.0f)
            {
               overkill_map[ALLEGRO_EVENT_KEY_DOWN] = al_map_rgb_f(0.0f, 1, 0.0f);
               overkill_map[ALLEGRO_EVENT_KEY_UP] = al_map_rgb_f(0.0f, 0.5, 0.0f);
               overkill_map[ALLEGRO_EVENT_TIMER] = al_map_rgb_f(1.0f, 0.0f, 1.0f);
               overkill_map[ALLEGRO_EVENT_MOUSE_AXES] = al_map_rgb_f(0.4f, 0.4f, 0.0f);
               overkill_map[ALLEGRO_EVENT_MOUSE_BUTTON_DOWN] = al_map_rgb_f(1.0f, 1.0f, 0.0f);
               overkill_map[ALLEGRO_EVENT_MOUSE_BUTTON_UP] = al_map_rgb_f(0.7f, 0.7f, 0.0f);
               overkill_map[ALLEGRO_EVENT_TIMER] = al_map_rgb_f(0.5f, 0.5, 0.5f);
               overkill_map[CUSTOM_EVENT_DRAWN] = al_map_rgb_f(0.0f, 1.0, 1.0f);
            }
      }events, random;

      class LogBar {
         private:
            const float width;
            const float half_scale;
            const float x_pos;
            const float y_pos;
            const ALLEGRO_COLOR red;
            const ALLEGRO_COLOR white;
            float current_scale_pos;
            float log_base;
         public:
            void set(float gpl) {
               //log scale with 0 as the center
               current_scale_pos = y_pos - half_scale - static_cast<float>(::log(gpl)) * log_base * half_scale;
               if(!finite(current_scale_pos))
                  current_scale_pos = 0.0f;
            }
            void draw() {
               al_draw_line (x_pos, current_scale_pos, x_pos + width, current_scale_pos, red, 2.0f);
               al_draw_line (x_pos, y_pos, x_pos + width, y_pos, white, 1.0f);
               al_draw_line (x_pos, y_pos - half_scale, x_pos + width, y_pos - half_scale, white, 1.0f);
               al_draw_line (x_pos, y_pos - half_scale * 2.0f, x_pos + width, y_pos - half_scale * 2.0f, white, 1.0f);
            }
            LogBar (float xpos, float ypos, float ysize, float logbase)
               : width(64.0f), half_scale(ysize/2.0f), x_pos(xpos), y_pos(ypos),
                 red(al_map_rgb_f(1.0f, 0.0f, 0.0f)), white(al_map_rgb_f(1.0f, 1.0f, 1.0f)),
                 current_scale_pos(ypos), log_base( 1.0f/::log(logbase) )

            {}
      }log;

      class LinearBar {
         private:
            const float width;
            const float scale_factor;
            const float scale_half;
            const float x_pos;
            const float y_pos;
            const ALLEGRO_COLOR red;
            const ALLEGRO_COLOR white;
            float current_scale_pos;
         public:
            void set(float gpl) {
               current_scale_pos = y_pos - gpl * scale_factor;
            }
            void draw() {
               al_draw_line (x_pos, current_scale_pos, x_pos + width, current_scale_pos, red, 2.0f);
               al_draw_line (x_pos, y_pos, x_pos + width, y_pos, white, 1.0f);
               al_draw_line (x_pos, y_pos + scale_half, x_pos + width, y_pos + scale_half, white, 1.0f);
               al_draw_line (x_pos, y_pos + scale_half * 2.0f, x_pos + width, y_pos + scale_half * 2.0f, white, 1.0f);
            }
            LinearBar (float xpos, float ypos, float ysize, float scalemax)
               : width(64.0f), scale_factor(ysize/scalemax), scale_half(scalemax/2.0f),
                 x_pos(xpos), y_pos(ypos), red(al_map_rgb_f(1.0f, 0.0f, 0.0f)),
                 white(al_map_rgb_f(1.0f, 1.0f, 1.0f)), current_scale_pos(0.0f)
            {}
      }lin, timer;

      class PerfBar {
         private:
            const float width;
            const float x_pos;
            const float x_end;
            const float y_pos;
            const float scale_height;
            const ALLEGRO_COLOR col_gtu;
            const ALLEGRO_COLOR col_ltu;
            const ALLEGRO_COLOR col_over_drive;
            const ALLEGRO_COLOR col_scale;
            float y_gtu;
            float y_ltu;
            bool over;
         public:
            void set(float gtu, float ltu, float dt) {
               y_gtu += (gtu/dt) * scale_height;
               y_ltu += (ltu/dt) * scale_height;
               over = false;
               if((y_ltu + y_ltu) > scale_height)
                  over = true;
            }
            void draw() {
               al_draw_filled_rectangle(x_pos, y_pos - (y_gtu + y_ltu) , x_pos + width, y_pos - y_gtu, col_ltu);
               al_draw_filled_rectangle(x_pos, y_pos - y_gtu, x_pos + width, y_pos, col_gtu);
               al_draw_line(x_pos - 4.0f, y_pos, x_pos + width + 4.0f, y_pos, col_scale, 1.0f);
               al_draw_line(x_pos - 4.0f, y_pos - scale_height/2, x_pos + width + 4.0f, y_pos - scale_height/2, col_scale, 1.0f);
               al_draw_line(x_pos - 4.0f, y_pos - scale_height, x_pos + width + 4.0f, y_pos - scale_height, col_scale, 1.0f);
               if(over){
                  al_draw_rectangle(x_pos - 1.0f, y_pos - scale_height, x_pos + width + 1.0f, y_pos - (y_gtu + y_ltu), col_over_drive, 1.0f);
               }
               y_gtu = y_ltu = 0.0f;
            }
            PerfBar(float x, float y, float height)
               : width(64.0f), x_pos(x), x_end(x+width), y_pos(y), scale_height(height),
               col_gtu(al_map_rgb_f(0.25f, 0.75f, 0.75f)), col_ltu(al_map_rgb_f(0.75f, 0.25f, 0.25f)),
               col_over_drive(al_map_rgb_f(1.0f, 0.75f, 0.25f)), col_scale(al_map_rgb_f(1.0f, 1.0f, 1.0f)),
               y_gtu(0.0f),y_ltu(0.0f), over(false)
            {}
      }perfbar;
   public:
      void logic(ALLEGRO_EVENT* e) {
         events.set(e);
         random.set(e); random.set(&drawn);
         if(e->type == ALLEGRO_EVENT_KEY_DOWN)
            switch (e->keyboard.keycode) {
               case ALLEGRO_KEY_F5: to_rest = 0.0f; break;
               case ALLEGRO_KEY_F6: to_rest = 0.001f * dt; break;
               case ALLEGRO_KEY_F7: to_rest = 0.01f * dt; break;
               case ALLEGRO_KEY_F8: to_rest = 0.1f * dt; break;
               case ALLEGRO_KEY_F9: to_rest = gtu; break;
               case ALLEGRO_KEY_F10: to_rest = gtu*2.0f; break;
               default: break;
            }
         if(e->type == ALLEGRO_EVENT_TIMER){
            if(log_not_lin)
               log.set(gpl);
            else
               lin.set(gpl);
            timer.set(time);
            time += dt;
            while(time >= 1.0f)
              time -= 2.0f;
            perfbar.set(gtu, ltu, dt);
            al_rest(to_rest);
         }
      }

      void draw() {
         al_clear_to_color(al_map_rgb_f(0.0f, 0.0f, 0.0f));
         events.draw();
         random.draw();
         if(log_not_lin)
            log.draw();
         else
            lin.draw();
         perfbar.draw();
         timer.draw();
         al_emit_user_event(&esrc, &drawn, NULL);
      }

      TestGame(ALLEGRO_EVENT_QUEUE* q, double& igtu, double& iltu, double& igpl, double& idt, bool use_log)
         : gtu(igtu), ltu(iltu), gpl(igpl), dt(idt), time(0.0f), to_rest(0.0f),
           log_not_lin(use_log),
           events(639.0f, 399.0f, 4.0f),
           random(639.0f, 440.0f, 16.0f),
           log(10.0f, 399.0f, 399.0f, 2.0f),
           lin(10.0f, 399.0f, 150.0f, 1.0f),
           timer(572.0f, 169.0f, 150.0f, 1.0f),
           perfbar(90.0f, 319.0f, 50.0f)
      {
         al_init_user_event_source(&esrc);
         al_register_event_source(q, &esrc);
         drawn.user.type = CUSTOM_EVENT_DRAWN;
      }
      ~TestGame(){
         al_destroy_user_event_source(&esrc);
      }
};
